@@ -1,5 +1,6 @@
-/*
+﻿/*
  *  Copyright 2011-2014  Andreas Cord-Landwehr <cordlandwehr@kde.org>
+ *  Copyright 2019       Caio Henrique Segawa Tonetti <caio.tonetti@gmail.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -34,8 +35,10 @@
 #include <QMap>
 #include <QPair>
 #include <QButtonGroup>
+#include <QMessageBox>
 
 #include <cmath>
+#include <random>
 
 #include <boost/graph/adjacency_list.hpp>
 #include <boost/graph/iteration_macros.hpp>
@@ -83,6 +86,7 @@ GenerateGraphWidget::GenerateGraphWidget(GraphDocumentPtr document, QWidget *par
     m_defaultIdentifiers.insert(CircleGraph, "CircleGraph");
     m_defaultIdentifiers.insert(ErdosRenyiRandomGraph, "RandomGraph");
     m_defaultIdentifiers.insert(RandomTree, "RandomTree");
+    m_defaultIdentifiers.insert(RandomDag, "RandomDag");
     m_defaultIdentifiers.insert(PathGraph, "PathGraph");
     m_defaultIdentifiers.insert(CompleteGraph, "CompleteGraph");
     m_defaultIdentifiers.insert(CompleteBipartiteGraph, "CompleteBipartite");
@@ -125,6 +129,8 @@ GenerateGraphWidget::GenerateGraphWidget(GraphDocumentPtr document, QWidget *par
     ui->GNPGeneratorSeed->setVisible(false);
     ui->label_randomTreeGeneratorSeed->setVisible(false);
     ui->randomTreeGeneratorSeed->setVisible(false);
+    ui->label_dagGeneratorSeed->setVisible(false);
+    ui->dagGeneratorSeed->setVisible(false);
 
     for (int i = 0; i < document->edgeTypes().length(); ++i) {
         EdgeTypePtr type = document->edgeTypes().at(i);
@@ -210,7 +216,7 @@ void GenerateGraphWidget::generateGraph()
         );
         break;
     case ErdosRenyiRandomGraph:
-        setSeed(ui->randomGeneratorSeed->value());
+        setSeed(ui->GNPGeneratorSeed->value());
         generateErdosRenyiRandomGraph(
             ui->GNPNodes->value(),
             ui->GNPEdgeProbability->value(),
@@ -223,6 +229,13 @@ void GenerateGraphWidget::generateGraph()
             ui->randomTreeNodes->value()
         );
         break;
+    case RandomDag:
+        setSeed(ui->dagGeneratorSeed->value());
+        generateRandomDagGraph(
+            ui->randomDagNumberOfNodes->value(),
+            ui->randomDagEdgeProbability->value()
+        );
+        break;
     case PathGraph:
         generatePathGraph(
             ui->pathNodes->value()
@@ -232,6 +245,7 @@ void GenerateGraphWidget::generateGraph()
         generateCompleteGraph(
             ui->completeNodes->value()
         );
+        break;
     case CompleteBipartiteGraph:
         generateCompleteBipartiteGraph(
             ui->completeBipartiteNodesLeft->value(),
@@ -256,10 +270,12 @@ QPointF GenerateGraphWidget::documentCenter() const
     qreal xSum = 0;
     qreal ySum = 0;
     int number = m_document->nodes().length();
+
     foreach (NodePtr node, m_document->nodes()) {
         xSum += node->x();
         ySum += node->y();
     }
+
     if (number > 0) {
         center.setX(xSum / number);
         center.setY(ySum / number);
@@ -267,6 +283,40 @@ QPointF GenerateGraphWidget::documentCenter() const
     return center;
 }
 
+/** @brief make sure all @p nodes are on the canvas
+ * 
+ * Given a collection of @p nodes, make sure they are all visible
+ * on the canvas (e.g. have a non-negative X and Y coordinate)
+ * by finding the smallest X and Y and then uniformly translating
+ * all of the @p nodes by that amount if either are negative.
+ */
+template<typename T>
+void adjustNodesToCanvas(T& nodes)
+{
+    qreal minX = 0;
+    qreal minY = 0;
+    for(auto& n : nodes) {
+        if (n->x() < minX) {
+            minX = n->x();
+        }
+        if (n->y() < minY) {
+            minY = n->y();
+        }
+    }
+    
+    if ((minX < 0) || (minY < 0)) {
+        // If either is non-negative, make it zero to not translate on that axis
+        minX = qMin(minX, qreal(0));
+        minY = qMin(minY, qreal(0));
+        
+        // min* is negative or zero, so subtracting it means **adding** to the coordinate
+        for(auto& n : nodes) {
+            n->setX(n->x() - minX);
+            n->setY(n->y() - minY);
+        }
+    }
+}
+
 void GenerateGraphWidget::generateMesh(int rows, int columns)
 {
     QPointF center = documentCenter();
@@ -292,6 +342,8 @@ void GenerateGraphWidget::generateMesh(int rows, int columns)
         }
     }
 
+    adjustNodesToCanvas(meshNodes);
+    
     // connect mesh nodes
     for (int i = 0; i < columns; ++i) {
         for (int j = 0; j < rows; ++j) {
@@ -341,6 +393,8 @@ void GenerateGraphWidget::generateStar(int satelliteNodes)
     node->setType(m_nodeType);
     nodes.prepend(node);
 
+    adjustNodesToCanvas(nodes);
+    
     // connect circle nodes
     for (int i = 1; i <= satelliteNodes; ++i) {
         EdgePtr edge = Edge::create(nodes.at(0), nodes.at(i));
@@ -367,6 +421,8 @@ void GenerateGraphWidget::generateCircle(int number)
         node->setType(m_nodeType);
         nodes.append(node);
     }
+    
+    adjustNodesToCanvas(nodes);
 
     // connect circle nodes
     for (int i = 0; i < number - 1; i++) {
@@ -382,11 +438,11 @@ void GenerateGraphWidget::generateRandomGraph(int nodes, int edges, bool selfEdg
     QPointF center = documentCenter();
 
     Graph randomGraph;
-    boost::mt19937 gen;
+    std::mt19937 gen;
     gen.seed(static_cast<unsigned int>(m_seed));
 
     // generate graph
-    boost::generate_random_graph<Graph, boost::mt19937>(
+    boost::generate_random_graph<Graph, std::mt19937>(
         randomGraph,
         nodes,
         edges,
@@ -395,14 +451,14 @@ void GenerateGraphWidget::generateRandomGraph(int nodes, int edges, bool selfEdg
     );
 
     // generate distribution topology and apply
-    boost::rectangle_topology< boost::mt19937 > topology(gen, center.x() - 20 * nodes, center.y() - 20 * nodes, center.x() + 20 * nodes, center.y() + 20 * nodes);
+    boost::rectangle_topology< std::mt19937 > topology(gen, center.x() - 20 * nodes, center.y() - 20 * nodes, center.x() + 20 * nodes, center.y() + 20 * nodes);
     PositionVec position_vec(boost::num_vertices(randomGraph));
     PositionMap positionMap(position_vec.begin(), get(boost::vertex_index, randomGraph));
 
     boost::random_graph_layout(randomGraph, positionMap, topology);
 
     // minimize cuts by Fruchtman-Reingold layout algorithm
-    boost::fruchterman_reingold_force_directed_layout< boost::rectangle_topology< boost::mt19937 >, Graph, PositionMap >
+    boost::fruchterman_reingold_force_directed_layout< boost::rectangle_topology< std::mt19937 >, Graph, PositionMap >
     (randomGraph,
      positionMap,
      topology,
@@ -419,6 +475,8 @@ void GenerateGraphWidget::generateRandomGraph(int nodes, int edges, bool selfEdg
         mapNodes[*vi]->setType(m_nodeType);
     }
 
+    adjustNodesToCanvas(mapNodes);
+    
     boost::graph_traits<Graph>::edge_iterator ei, ei_end;
     for (boost::tie(ei, ei_end) = boost::edges(randomGraph); ei != ei_end; ++ei) {
         EdgePtr edge = Edge::create(mapNodes[boost::source(*ei, randomGraph)], mapNodes[boost::target(*ei, randomGraph)]);
@@ -430,21 +488,21 @@ void GenerateGraphWidget::generateErdosRenyiRandomGraph(int nodes, double edgePr
 {
     QPointF center = documentCenter();
 
-    boost::mt19937 gen;
+    std::mt19937 gen;
     gen.seed(static_cast<unsigned int>(m_seed));
 
     // generate graph
-    typedef boost::erdos_renyi_iterator<boost::mt19937, Graph> ergen;
+    typedef boost::erdos_renyi_iterator<std::mt19937, Graph> ergen;
     Graph randomGraph(ergen(gen, nodes, edgeProbability, selfEdges), ergen(), nodes);
 
     // generate distribution topology and apply
-    boost::rectangle_topology< boost::mt19937 > topology(gen, center.x() - 20 * nodes, center.y() - 20 * nodes, center.x() + 20 * nodes, center.y() + 20 * nodes);
+    boost::rectangle_topology< std::mt19937 > topology(gen, center.x() - 20 * nodes, center.y() - 20 * nodes, center.x() + 20 * nodes, center.y() + 20 * nodes);
     PositionVec position_vec(boost::num_vertices(randomGraph));
     PositionMap positionMap(position_vec.begin(), get(boost::vertex_index, randomGraph));
     boost::random_graph_layout(randomGraph, positionMap, topology);
 
     // minimize cuts by Fruchtman-Reingold layout algorithm
-    boost::fruchterman_reingold_force_directed_layout< boost::rectangle_topology< boost::mt19937 >, Graph, PositionMap >
+    boost::fruchterman_reingold_force_directed_layout< boost::rectangle_topology< std::mt19937 >, Graph, PositionMap >
     (randomGraph,
      positionMap,
      topology,
@@ -461,6 +519,8 @@ void GenerateGraphWidget::generateErdosRenyiRandomGraph(int nodes, double edgePr
         mapNodes[*vi]->setType(m_nodeType);
     }
 
+    adjustNodesToCanvas(mapNodes);
+    
     boost::graph_traits<Graph>::edge_iterator ei, ei_end;
     for (boost::tie(ei, ei_end) = boost::edges(randomGraph); ei != ei_end; ++ei) {
         EdgePtr edge = Edge::create(mapNodes[boost::source(*ei, randomGraph)], mapNodes[boost::target(*ei, randomGraph)]);
@@ -470,31 +530,86 @@ void GenerateGraphWidget::generateErdosRenyiRandomGraph(int nodes, double edgePr
 
 void GenerateGraphWidget::generateRandomTreeGraph(int number)
 {
-    boost::mt19937 gen;
+
+    if (EdgeType::Unidirectional == m_edgeType->direction()){
+        QMessageBox::critical(this, "Incorrect Edge Direction", "Edges in a Tree must be bidirectional.");
+        return;
+    }
+
+    std::mt19937 gen;
     gen.seed(static_cast<unsigned int>(m_seed));
 
     NodeList nodes;
 
-    NodePtr node = Node::create(m_document);
-    node->setType(m_nodeType);
-    nodes.append(node);
+    QVector<int> notAdded;
+    QVector<int> added;
 
-    for (int i = 1; i < number; ++i) {
-        NodePtr thisNode = Node::create(m_document);
+    for (int i = 0; i < number; i++) {
+        NodePtr node = Node::create(m_document);
         node->setType(m_nodeType);
-        boost::random::uniform_int_distribution<> randomEarlierNodeGen(0, i-1);
-        int randomEarlierNode = randomEarlierNodeGen(gen);
-        EdgePtr edge = Edge::create(thisNode, nodes.at(randomEarlierNode));
-        edge->setType(m_edgeType);
-        if (m_edgeType->direction() == EdgeType::Unidirectional) {
-            edge = Edge::create(nodes.at(randomEarlierNode), thisNode);
-            edge->setType(m_edgeType);
-        }
-        nodes.append(thisNode);
+        nodes.append(node);
+
+        notAdded.push_back(i);
     }
 
-    Topology topology = Topology();
-    topology.directedGraphDefaultTopology(m_document);
+    // shuffle
+    std::shuffle(notAdded.begin(), notAdded.end(), gen);
+
+    // add root
+    added.push_back(notAdded.front());
+    notAdded.pop_front();
+
+    while (!notAdded.empty()) {
+        boost::random::uniform_int_distribution<> dist(0, added.size()-1);
+
+        int randomIdx = dist(gen);
+        int next = notAdded.front();
+
+        notAdded.pop_front();
+        added.push_back(next);
+
+        EdgePtr edge = Edge::create(nodes.at(added[randomIdx]), nodes.at(next));
+        edge->setType(m_edgeType);
+    }
+
+    Topology::applyCircleAlignment(nodes, 300);
+    Topology::applyMinCutTreeAlignment(nodes);
+    adjustNodesToCanvas(nodes);
+}
+
+void GenerateGraphWidget::generateRandomDagGraph(int nodes, double edgeProbability)
+{
+
+    if (EdgeType::Bidirectional == m_edgeType->direction()){
+        QMessageBox::critical(this, i18n("Incorrect Edge Direction"), i18n("Edges in a Directed Acyclical Graph must be directional."));
+        return;
+    }
+
+    std::mt19937 gen;
+    gen.seed(static_cast<unsigned int>(m_seed));
+    boost::random::uniform_real_distribution<double> dist(0, 1);
+
+    NodeList nodes_list;
+
+    for (int j=0; j < nodes; j++) {
+        NodePtr node = Node::create(m_document);
+        node->setType(m_nodeType);
+        nodes_list.append(node);
+    }
+
+    // Create random edges
+    for (int i=0; i < nodes - 1; i++) {
+        for (int j=i+1; j < nodes; j++) {
+            if (dist(gen)< edgeProbability) {
+                EdgePtr edge = Edge::create(nodes_list.at(i), nodes_list.at(j));
+                edge->setType(m_edgeType);
+            }
+        }
+    }
+
+    Topology::applyCircleAlignment(nodes_list, 300);
+    Topology::applyMinCutTreeAlignment(nodes_list);
+    adjustNodesToCanvas(nodes_list);
 }
 
 void GenerateGraphWidget::generatePathGraph(int pathSize)
@@ -503,7 +618,6 @@ void GenerateGraphWidget::generatePathGraph(int pathSize)
 
     QList< QPair<QString, QPointF> > pathNodes;
 
-
     NodeList nodes_list;
     for (int i = 1; i <= pathSize; i++) {
         NodePtr node = Node::create(m_document);
@@ -513,6 +627,8 @@ void GenerateGraphWidget::generatePathGraph(int pathSize)
         nodes_list.append(node);
     }
 
+    adjustNodesToCanvas(nodes_list);
+    
     for (int i = 0; i < pathSize - 1; i++) {
         EdgePtr edge = Edge::create(nodes_list.at(i), nodes_list.at(i + 1));
         edge->setType(m_edgeType);
@@ -538,8 +654,10 @@ void GenerateGraphWidget::generateCompleteGraph(int nodes)
         node_list.append(node);
     }
 
+    adjustNodesToCanvas(node_list);
+    
     for (int i = 0; i < nodes - 1; i++) {
-        for (int j = i + 1; j < nodes; j++){
+        for (int j = i + 1; j < nodes; j++) {
             EdgePtr edge_lr = Edge::create(node_list.at(i), node_list.at(j));
             edge_lr->setType(m_edgeType);
 
@@ -574,6 +692,8 @@ void GenerateGraphWidget::generateCompleteBipartiteGraph(int nodes_left, int nod
         node_list.append(node);
     }
 
+    adjustNodesToCanvas(node_list);
+
     for (int i = 0; i < nodes_left; i++) {
         for (int j = 0; j < nodes_right; j++){
             EdgePtr edge_lr = Edge::create(node_list.at(i), node_list.at(j + nodes_left));
